#!/usr/bin/env python
#
#  mod_authz_svn_tests.py:  testing mod_authz_svn
#
#  Subversion is a tool for revision control.
#  See http://subversion.apache.org for more information.
#
# ====================================================================
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
######################################################################

# General modules
import os, re, logging

logger = logging.getLogger()

# Our testing module
import svntest

# (abbreviation)
Skip = svntest.testcase.Skip_deco
SkipUnless = svntest.testcase.SkipUnless_deco
XFail = svntest.testcase.XFail_deco
Issues = svntest.testcase.Issues_deco
Issue = svntest.testcase.Issue_deco
Wimp = svntest.testcase.Wimp_deco

ls_of_D_no_H = '''<html><head><title>repos - Revision 1: /A/D</title></head>
<body>
 <h2>repos - Revision 1: /A/D</h2>
 <ul>
  <li><a href="../">..</a></li>
  <li><a href="G/">G/</a></li>
  <li><a href="gamma">gamma</a></li>
 </ul>
</body></html>'''

ls_of_D_H = '''<html><head><title>repos - Revision 1: /A/D</title></head>
<body>
 <h2>repos - Revision 1: /A/D</h2>
 <ul>
  <li><a href="../">..</a></li>
  <li><a href="G/">G/</a></li>
  <li><a href="H/">H/</a></li>
  <li><a href="gamma">gamma</a></li>
 </ul>
</body></html>'''

ls_of_H = '''<html><head><title>repos - Revision 1: /A/D/H</title></head>
<body>
 <h2>repos - Revision 1: /A/D/H</h2>
 <ul>
  <li><a href="../">..</a></li>
  <li><a href="chi">chi</a></li>
  <li><a href="omega">omega</a></li>
  <li><a href="psi">psi</a></li>
 </ul>
</body></html>'''

user1 = svntest.main.wc_author
user1_upper = user1.upper()
user1_pass = svntest.main.wc_passwd
user1_badpass = 'XXX'
assert user1_pass != user1_badpass, "Passwords can't match"
user2 = svntest.main.wc_author2
user2_upper = user2.upper()
user2_pass = svntest.main.wc_passwd
user2_badpass = 'XXX'
assert user2_pass != user2_badpass, "Passwords can't match"

def write_authz_file(sbox):
    svntest.main.write_authz_file(sbox, {
                                          '/':  '$anonymous = r\n' +
                                                'jrandom = rw\n' +
                                                'jconstant = rw',
                                          '/A/D/H': '$anonymous =\n' +
                                                    '$authenticated =\n' +
                                                    'jrandom = rw'
                                        })

def write_authz_file_groups(sbox):
    authz_name = sbox.authz_name()
    svntest.main.write_authz_file(sbox,{
                                         '/':  '* =',
                                       })

def verify_get(test_area_url, path, user, pw,
               expected_status, expected_body, headers):
  import httplib
  from urlparse import urlparse
  import base64

  req_url = test_area_url + path

  loc = urlparse(req_url)

  if loc.scheme == 'http':
    h = httplib.HTTPConnection(loc.hostname, loc.port)
  else:
    h = httplib.HTTPSConnection(loc.hostname, loc.port)

  if headers is None:
    headers = {}

  if user and pw:
      auth_info = user + ':' + pw
      headers['Authorization'] = 'Basic ' + base64.b64encode(auth_info)
  else:
      auth_info = "anonymous"

  h.request('GET', req_url, None, headers)

  r = h.getresponse()

  actual_status = r.status
  if expected_status and expected_status != actual_status:

      logger.warn("Expected status '" + str(expected_status) +
                  "' but got '" + str(actual_status) +
                  "' on url '" + req_url + "' (" +
                  auth_info + ").")
      raise svntest.Failure

  if expected_body:
      actual_body = r.read()
      if expected_body != actual_body:
        logger.warn("Expected body:")
        logger.warn(expected_body)
        logger.warn("But got:")
        logger.warn(actual_body)
        logger.warn("on url '" + req_url + "' (" + auth_info + ").")
        raise svntest.Failure

def verify_gets(test_area_url, tests):
  for test in tests:
      verify_get(test_area_url, test['path'], test.get('user'), test.get('pw'),
                 test['status'], test.get('body'), test.get('headers'))


######################################################################
# Tests
#
#   Each test must return on success or raise on failure.


#----------------------------------------------------------------------


@SkipUnless(svntest.main.is_ra_type_dav)
def anon(sbox):
  "test anonymous access"
  sbox.build(read_only = True, create_wc = False)

  test_area_url = sbox.repo_url.replace('/svn-test-work/local_tmp/repos',
                                        '/authz-test-work/anon')

  write_authz_file(sbox)

  anon_tests = ( 
                 { 'path': '', 'status': 301 },
                 { 'path': '/', 'status': 200 },
                 { 'path': '/repos', 'status': 301 },
                 { 'path': '/repos/', 'status': 200 },
                 { 'path': '/repos/A', 'status': 301 },
                 { 'path': '/repos/A/', 'status': 200 },
                 { 'path': '/repos/A/D', 'status': 301 },
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H },
                 { 'path': '/repos/A/D/gamma', 'status': 200 },
                 { 'path': '/repos/A/D/H', 'status': 403 },
                 { 'path': '/repos/A/D/H/', 'status': 403 },
                 { 'path': '/repos/A/D/H/chi', 'status': 403 },
                 # auth isn't configured so nothing should change when passing
                 # authn details
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_badpass},
               )

  verify_gets(test_area_url, anon_tests)


@SkipUnless(svntest.main.is_ra_type_dav)
def mixed(sbox):
  "test mixed anonymous and authenticated access"
  sbox.build(read_only = True, create_wc = False)

  test_area_url = sbox.repo_url.replace('/svn-test-work/local_tmp/repos',
                                        '/authz-test-work/mixed')

  write_authz_file(sbox)

  mixed_tests = (
                 { 'path': '', 'status': 301,  },
                 { 'path': '/', 'status': 200,  },
                 { 'path': '/repos', 'status': 301,  },
                 { 'path': '/repos/', 'status': 200,  },
                 { 'path': '/repos/A', 'status': 301,  },
                 { 'path': '/repos/A/', 'status': 200,  },
                 { 'path': '/repos/A/D', 'status': 301,  },
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   },
                 { 'path': '/repos/A/D/gamma', 'status': 200, },
                 { 'path': '/repos/A/D/H', 'status': 401, },
                 { 'path': '/repos/A/D/H/', 'status': 401, },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, },
                 # auth is configured and user1 is allowed access to H
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_H,
                   'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 200, 'body': ls_of_H, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 200, 'user': user1, 'pw': user1_pass},
                 # try with the wrong password for user1
                 { 'path': '', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 # auth is configured and user2 is not allowed access to H
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_pass},
                 # try with the wrong password for user2
                 { 'path': '', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 )

  verify_gets(test_area_url, mixed_tests)

@SkipUnless(svntest.main.is_ra_type_dav)
@XFail(svntest.main.is_httpd_authz_provider_enabled)
# uses the AuthzSVNNoAuthWhenAnonymousAllowed On directive
# this is broken with httpd 2.3.x+ since it requires the auth system to accept
# r->user == NULL and there is a test for this in server/request.c now.  It
# was intended as a workaround for the lack of Satisfy Any in 2.3.x+ which
# was resolved by httpd with mod_access_compat in 2.3.x+.
def mixed_noauthwhenanon(sbox):
  "test mixed with noauthwhenanon directive"
  sbox.build(read_only = True, create_wc = False)

  test_area_url = sbox.repo_url.replace('/svn-test-work/local_tmp/repos',
                                        '/authz-test-work/mixed-noauthwhenanon')

  write_authz_file(sbox)

  noauthwhenanon_tests = (
                 { 'path': '', 'status': 301,  },
                 { 'path': '/', 'status': 200,  },
                 { 'path': '/repos', 'status': 301,  },
                 { 'path': '/repos/', 'status': 200,  },
                 { 'path': '/repos/A', 'status': 301,  },
                 { 'path': '/repos/A/', 'status': 200,  },
                 { 'path': '/repos/A/D', 'status': 301,  },
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   },
                 { 'path': '/repos/A/D/gamma', 'status': 200, },
                 { 'path': '/repos/A/D/H', 'status': 401, },
                 { 'path': '/repos/A/D/H/', 'status': 401, },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, },
                 # auth is configured and user1 is allowed access to H
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_H,
                   'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 200, 'body': ls_of_H, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 200, 'user': user1, 'pw': user1_pass},
                 # try with the wrong password for user1
                 # note that unlike doing this with Satisfy Any this case
                 # actually provides anon access when provided with an invalid
                 # password
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/', 'status': 200, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 # auth is configured and user2 is not allowed access to H
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_pass},
                 # try with the wrong password for user2
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/', 'status': 200, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 )

  verify_gets(test_area_url, noauthwhenanon_tests)


@SkipUnless(svntest.main.is_ra_type_dav)
def authn(sbox):
  "test authenticated only access"
  sbox.build(read_only = True, create_wc = False)

  test_area_url = sbox.repo_url.replace('/svn-test-work/local_tmp/repos',
                                        '/authz-test-work/authn')

  write_authz_file(sbox)

  authn_tests = (
                 { 'path': '', 'status': 401,  },
                 { 'path': '/', 'status': 401,  },
                 { 'path': '/repos', 'status': 401,  },
                 { 'path': '/repos/', 'status': 401,  },
                 { 'path': '/repos/A', 'status': 401,  },
                 { 'path': '/repos/A/', 'status': 401,  },
                 { 'path': '/repos/A/D', 'status': 401,  },
                 { 'path': '/repos/A/D/', 'status': 401, },
                 { 'path': '/repos/A/D/gamma', 'status': 401, },
                 { 'path': '/repos/A/D/H', 'status': 401, },
                 { 'path': '/repos/A/D/H/', 'status': 401, },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, },
                 # auth is configured and user1 is allowed access to H
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_H,
                   'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 200, 'body': ls_of_H, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 200, 'user': user1, 'pw': user1_pass},
                 # try with upper case username for user1
                 { 'path': '', 'status': 301, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 # try with the wrong password for user1
                 { 'path': '', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 # auth is configured and user2 is not allowed access to H
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_pass},
                 # try with upper case username for user2
                 { 'path': '', 'status': 301, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/', 'status': 200, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 # try with the wrong password for user2
                 { 'path': '', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 )

  verify_gets(test_area_url, authn_tests)

@SkipUnless(svntest.main.is_ra_type_dav)
def authn_anonoff(sbox):
  "test authenticated only access with anonoff"
  sbox.build(read_only = True, create_wc = False)

  test_area_url = sbox.repo_url.replace('/svn-test-work/local_tmp/repos',
                                        '/authz-test-work/authn-anonoff')

  write_authz_file(sbox)

  anonoff_tests = (
                 { 'path': '', 'status': 401,  },
                 { 'path': '/', 'status': 401,  },
                 { 'path': '/repos', 'status': 401,  },
                 { 'path': '/repos/', 'status': 401,  },
                 { 'path': '/repos/A', 'status': 401,  },
                 { 'path': '/repos/A/', 'status': 401,  },
                 { 'path': '/repos/A/D', 'status': 401,  },
                 { 'path': '/repos/A/D/', 'status': 401, },
                 { 'path': '/repos/A/D/gamma', 'status': 401, },
                 { 'path': '/repos/A/D/H', 'status': 401, },
                 { 'path': '/repos/A/D/H/', 'status': 401, },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, },
                 # auth is configured and user1 is allowed access to H
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_H,
                   'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 200, 'body': ls_of_H, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 200, 'user': user1, 'pw': user1_pass},
                 # try with upper case username for user1
                 { 'path': '', 'status': 301, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user1_upper, 'pw': user1_pass},
                 # try with the wrong password for user1
                 { 'path': '', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 # auth is configured and user2 is not allowed access to H
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_pass},
                 # try with upper case username for user2
                 { 'path': '', 'status': 301, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/', 'status': 200, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 # try with the wrong password for user2
                 { 'path': '', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 )

  verify_gets(test_area_url, anonoff_tests)

@SkipUnless(svntest.main.is_ra_type_dav)
def authn_lcuser(sbox):
  "test authenticated only access with lcuser"
  sbox.build(read_only = True, create_wc = False)

  test_area_url = sbox.repo_url.replace('/svn-test-work/local_tmp/repos',
                                        '/authz-test-work/authn-lcuser')

  write_authz_file(sbox)

  lcuser_tests = (
                 # try with upper case username for user1 (works due to lcuser option)
                 { 'path': '', 'status': 301, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 301, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_H,
                   'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 301, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 200, 'body': ls_of_H, 'user': user1_upper, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 200, 'user': user1_upper, 'pw': user1_pass},
                 # try with upper case username for user2 (works due to lcuser option)
                 { 'path': '', 'status': 301, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/', 'status': 200, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos', 'status': 301, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2_upper, 'pw': user2_pass},
                 )

  verify_gets(test_area_url, lcuser_tests)

# authenticated access only by group - a excuse to use AuthzSVNAuthoritative Off
# this is terribly messed up, Require group runs after mod_authz_svn.
# so if mod_authz_svn grants the access then it doesn't matter what the group
# requirement says.  If we reject the access then you can use the AuthzSVNAuthoritative Off
# directive to fall through to the group check.  Overall the behavior of setups like this
# is almost guaranteed to not be what users expect.
@SkipUnless(svntest.main.is_ra_type_dav)
def authn_group(sbox):
  "test authenticated only access via groups"
  sbox.build(read_only = True, create_wc = False)

  test_area_url = sbox.repo_url.replace('/svn-test-work/local_tmp/repos',
                                        '/authz-test-work/authn-group')

  # Can't use write_authz_file() as most tests because we want to deny all
  # access with mod_authz_svn so the tests fall through to the group handling
  authz_name = sbox.authz_name()
  svntest.main.write_authz_file(sbox, {
                                        '/':  '* =',
                                      })

  group_tests = (
                 { 'path': '', 'status': 401, },
                 { 'path': '/', 'status': 401, },
                 { 'path': '/repos', 'status': 401, },
                 { 'path': '/repos/', 'status': 401, },
                 { 'path': '/repos/A', 'status': 401, },
                 { 'path': '/repos/A/', 'status': 401, },
                 { 'path': '/repos/A/D', 'status': 401, },
                 { 'path': '/repos/A/D/', 'status': 401, },
                 { 'path': '/repos/A/D/gamma', 'status': 401, },
                 { 'path': '/repos/A/D/H', 'status': 401, },
                 { 'path': '/repos/A/D/H/', 'status': 401, },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, },
                 # auth is configured and user1 is allowed access repo including H
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_H,
                   'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 200, 'body': ls_of_H, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 200, 'user': user1, 'pw': user1_pass},
                 )

  verify_gets(test_area_url, group_tests)

# This test exists to validate our behavior when used with the new authz
# provider system introduced in httpd 2.3.x.  The Satisfy directive
# determines how older authz hooks are combined and the RequireA(ll|ny)
# blocks handles how new authz providers are combined.  The overall results of
# all the authz providers (combined per the Require* blocks) are then
# combined with the other authz hooks via the Satisfy directive.
# Meaning this test requires that mod_authz_svn says yes and there is
# either a valid user or the ALLOW header is 1.  The header may seem
# like a silly test but it's easier to excercise than say a host directive
# in a repeatable test.
@SkipUnless(svntest.main.is_httpd_authz_provider_enabled)
def authn_sallrany(sbox):
  "test satisfy all require any config"
  sbox.build(read_only = True, create_wc = False)

  test_area_url = sbox.repo_url.replace('/svn-test-work/local_tmp/repos',
                                        '/authz-test-work/sallrany')

  write_authz_file(sbox)

  allow_header = { 'ALLOW': '1' }

  sallrany_tests = (
                 #anon access isn't allowed without ALLOW header
                 { 'path': '', 'status': 401, },
                 { 'path': '/', 'status': 401, },
                 { 'path': '/repos', 'status': 401, },
                 { 'path': '/repos/', 'status': 401, },
                 { 'path': '/repos/A', 'status': 401, },
                 { 'path': '/repos/A/', 'status': 401, },
                 { 'path': '/repos/A/D', 'status': 401, },
                 { 'path': '/repos/A/D/', 'status': 401, },
                 { 'path': '/repos/A/D/gamma', 'status': 401, },
                 { 'path': '/repos/A/D/H', 'status': 401, },
                 { 'path': '/repos/A/D/H/', 'status': 401, },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, },
                 # auth is configured and user1 is allowed access repo including H
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_H,
                   'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 301, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 200, 'body': ls_of_H, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 200, 'user': user1, 'pw': user1_pass},
                 # try with the wrong password for user1
                 { 'path': '', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user1, 'pw': user1_badpass},
                 # auth is configured and user2 is not allowed access to H
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_pass},
                 # try with the wrong password for user2
                 { 'path': '', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user2, 'pw': user2_badpass},
                 # anon is allowed with the ALLOW header
                 { 'path': '', 'status': 301, 'headers': allow_header },
                 { 'path': '/', 'status': 200, 'headers': allow_header },
                 { 'path': '/repos', 'status': 301, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 200, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 301, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 200, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 301, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'headers': allow_header },
                 # these 3 tests return 403 instead of 401 becasue the config allows
                 # the anon user with the ALLOW header without any auth and the old hook
                 # system has no way of knowing it should return 401 since authentication is
                 # configured and can change the behavior.  It could decide to return 401 just on
                 # the basis of authentication being configured but then that leaks info in other
                 # cases so it's better for this case to be "broken".
                 { 'path': '/repos/A/D/H', 'status': 403, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 403, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'headers': allow_header },
                 # auth is configured and user1 is allowed access repo including H
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_H,
                   'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 200, 'body': ls_of_H, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 # try with the wrong password for user1
                 { 'path': '', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 # auth is configured and user2 is not allowed access to H
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 # try with the wrong password for user2
                 { 'path': '', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },

                 )

  verify_gets(test_area_url, sallrany_tests)

# See comments on authn_sallrany test for some background on the interaction
# of Satisfy Any and the newer Require blocks.
@SkipUnless(svntest.main.is_httpd_authz_provider_enabled)
def authn_sallrall(sbox):
  "test satisfy all require all config"
  sbox.build(read_only = True, create_wc = False)

  test_area_url = sbox.repo_url.replace('/svn-test-work/local_tmp/repos',
                                        '/authz-test-work/sallrall')

  write_authz_file(sbox)

  allow_header = { 'ALLOW': '1' }

  sallrall_tests = (
                 #anon access isn't allowed without ALLOW header
                 { 'path': '', 'status': 403, },
                 { 'path': '/', 'status': 403, },
                 { 'path': '/repos', 'status': 403, },
                 { 'path': '/repos/', 'status': 403, },
                 { 'path': '/repos/A', 'status': 403, },
                 { 'path': '/repos/A/', 'status': 403, },
                 { 'path': '/repos/A/D', 'status': 403, },
                 { 'path': '/repos/A/D/', 'status': 403, },
                 { 'path': '/repos/A/D/gamma', 'status': 403, },
                 { 'path': '/repos/A/D/H', 'status': 403, },
                 { 'path': '/repos/A/D/H/', 'status': 403, },
                 { 'path': '/repos/A/D/H/chi', 'status': 403, },
                 # auth is configured but no access is allowed without the ALLOW header
                 { 'path': '', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user1, 'pw': user1_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user1, 'pw': user1_pass},
                 # try with the wrong password for user1
                 { 'path': '', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user1, 'pw': user1_badpass},
                 # auth is configured but no access is allowed without the ALLOW header
                 { 'path': '', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/gamma', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_pass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_pass},
                 # try with the wrong password for user2
                 { 'path': '', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/gamma', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_badpass},
                 # anon is not allowed even with ALLOW header
                 { 'path': '', 'status': 401, 'headers': allow_header },
                 { 'path': '/', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos/A/D/H', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 401, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'headers': allow_header },
                 # auth is configured and user1 is allowed access repo including H
                 { 'path': '', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_H,
                   'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H', 'status': 301, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 200, 'body': ls_of_H, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 200, 'user': user1, 'pw': user1_pass, 'headers': allow_header },
                 # try with the wrong password for user1
                 { 'path': '', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user1, 'pw': user1_badpass, 'headers': allow_header },
                 # auth is configured and user2 is not allowed access to H
                 { 'path': '', 'status': 301, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/', 'status': 200, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos', 'status': 301, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 200, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 301, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 200, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 301, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 200, 'body': ls_of_D_no_H,
                   'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 200, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H', 'status': 403, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 403, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 403, 'user': user2, 'pw': user2_pass, 'headers': allow_header },
                 # try with the wrong password for user2
                 { 'path': '', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/gamma', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },
                 { 'path': '/repos/A/D/H/chi', 'status': 401, 'user': user2, 'pw': user2_badpass, 'headers': allow_header },

                 )

  verify_gets(test_area_url, sallrall_tests)


########################################################################
# Run the tests


# list all tests here, starting with None:
test_list = [ None,
              anon,
              mixed,
              mixed_noauthwhenanon,
              authn,
              authn_anonoff,
              authn_lcuser,
              authn_group,
              authn_sallrany,
              authn_sallrall,
             ]
serial_only = True

if __name__ == '__main__':
  svntest.main.run_tests(test_list)
  # NOTREACHED


### End of file.



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 2.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_diff', [dirname(__file__)])
        except ImportError:
            import _diff
            return _diff
        if fp is not None:
            try:
                _mod = imp.load_module('_diff', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _diff = swig_import_helper()
    del swig_import_helper
else:
    import _diff
del version_info
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

def _copy_metadata_deep(value, old_value):
  """Copy all attributes of old_value into value, recursively traversing
  lists and dicts if needed."""
  if value is None or old_value is None or value is old_value: return
  
  if isinstance(value, dict):
    for k, v in value.iteritems():
      _copy_metadata_deep(v, old_value[k])
  elif isinstance(value, list):
    for v, old_v in zip(value, old_value):
      _copy_metadata_deep(v, old_v)
  else:
    try:
      value.__dict__.update(old_value.__dict__)
    except AttributeError:
      pass
      
def _assert_valid_deep(value):
  """Assert value's validity, recursively traversing lists and dicts."""
  if isinstance(value, dict):
    for v in value.itervalues():
      _assert_valid_deep(v)
  elif isinstance(value, list):
    for v in value:
      _assert_valid_deep(v)
  else:
    if hasattr(value, "assert_valid"):
      value.assert_valid()

import libsvn.core

def svn_diff_version():
  """svn_diff_version() -> svn_version_t const *"""
  return _diff.svn_diff_version()
svn_diff_datasource_original = _diff.svn_diff_datasource_original
svn_diff_datasource_modified = _diff.svn_diff_datasource_modified
svn_diff_datasource_latest = _diff.svn_diff_datasource_latest
svn_diff_datasource_ancestor = _diff.svn_diff_datasource_ancestor
class svn_diff_fns2_t:
    """Proxy of C svn_diff_fns2_t struct"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, svn_diff_fns2_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, svn_diff_fns2_t, name)
    __repr__ = _swig_repr
    __swig_setmethods__["datasources_open"] = _diff.svn_diff_fns2_t_datasources_open_set
    __swig_getmethods__["datasources_open"] = _diff.svn_diff_fns2_t_datasources_open_get
    __swig_setmethods__["datasource_close"] = _diff.svn_diff_fns2_t_datasource_close_set
    __swig_getmethods__["datasource_close"] = _diff.svn_diff_fns2_t_datasource_close_get
    __swig_setmethods__["datasource_get_next_token"] = _diff.svn_diff_fns2_t_datasource_get_next_token_set
    __swig_getmethods__["datasource_get_next_token"] = _diff.svn_diff_fns2_t_datasource_get_next_token_get
    __swig_setmethods__["token_compare"] = _diff.svn_diff_fns2_t_token_compare_set
    __swig_getmethods__["token_compare"] = _diff.svn_diff_fns2_t_token_compare_get
    __swig_setmethods__["token_discard"] = _diff.svn_diff_fns2_t_token_discard_set
    __swig_getmethods__["token_discard"] = _diff.svn_diff_fns2_t_token_discard_get
    __swig_setmethods__["token_discard_all"] = _diff.svn_diff_fns2_t_token_discard_all_set
    __swig_getmethods__["token_discard_all"] = _diff.svn_diff_fns2_t_token_discard_all_get
    def set_parent_pool(self, parent_pool=None):
      """Create a new proxy object for svn_diff_fns2_t"""
      import libsvn.core, weakref
      self.__dict__["_parent_pool"] = \
        parent_pool or libsvn.core.application_pool;
      if self.__dict__["_parent_pool"]:
        self.__dict__["_is_valid"] = weakref.ref(
          self.__dict__["_parent_pool"]._is_valid)

    def assert_valid(self):
      """Assert that this object is using valid pool memory"""
      if "_is_valid" in self.__dict__:
        assert self.__dict__["_is_valid"](), "Variable has already been deleted"

    def __getattr__(self, name):
      """Get an attribute from this object"""
      self.assert_valid()

      value = _swig_getattr(self, self.__class__, name)



      members = self.__dict__.get("_members")
      if members is not None:
        _copy_metadata_deep(value, members.get(name))
          

      _assert_valid_deep(value)

      return value

    def __setattr__(self, name, value):
      """Set an attribute on this object"""
      self.assert_valid()




      self.__dict__.setdefault("_members",{})[name] = value

      return _swig_setattr(self, self.__class__, name, value)

    def datasources_open(self, *args):
      return svn_diff_fns2_invoke_datasources_open(self, *args)

    def datasource_close(self, *args):
      return svn_diff_fns2_invoke_datasource_close(self, *args)

    def datasource_get_next_token(self, *args):
      return svn_diff_fns2_invoke_datasource_get_next_token(self, *args)

    def token_compare(self, *args):
      return svn_diff_fns2_invoke_token_compare(self, *args)

    def token_discard(self, *args):
      return svn_diff_fns2_invoke_token_discard(self, *args)

    def token_discard_all(self, *args):
      return svn_diff_fns2_invoke_token_discard_all(self, *args)

    def __init__(self): 
        """__init__(svn_diff_fns2_t self) -> svn_diff_fns2_t"""
        this = _diff.new_svn_diff_fns2_t()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _diff.delete_svn_diff_fns2_t
    __del__ = lambda self : None;
svn_diff_fns2_t_swigregister = _diff.svn_diff_fns2_t_swigregister
svn_diff_fns2_t_swigregister(svn_diff_fns2_t)

class svn_diff_fns_t:
    """Proxy of C svn_diff_fns_t struct"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, svn_diff_fns_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, svn_diff_fns_t, name)
    __repr__ = _swig_repr
    __swig_setmethods__["datasource_open"] = _diff.svn_diff_fns_t_datasource_open_set
    __swig_getmethods__["datasource_open"] = _diff.svn_diff_fns_t_datasource_open_get
    __swig_setmethods__["datasource_close"] = _diff.svn_diff_fns_t_datasource_close_set
    __swig_getmethods__["datasource_close"] = _diff.svn_diff_fns_t_datasource_close_get
    __swig_setmethods__["datasource_get_next_token"] = _diff.svn_diff_fns_t_datasource_get_next_token_set
    __swig_getmethods__["datasource_get_next_token"] = _diff.svn_diff_fns_t_datasource_get_next_token_get
    __swig_setmethods__["token_compare"] = _diff.svn_diff_fns_t_token_compare_set
    __swig_getmethods__["token_compare"] = _diff.svn_diff_fns_t_token_compare_get
    __swig_setmethods__["token_discard"] = _diff.svn_diff_fns_t_token_discard_set
    __swig_getmethods__["token_discard"] = _diff.svn_diff_fns_t_token_discard_get
    __swig_setmethods__["token_discard_all"] = _diff.svn_diff_fns_t_token_discard_all_set
    __swig_getmethods__["token_discard_all"] = _diff.svn_diff_fns_t_token_discard_all_get
    def set_parent_pool(self, parent_pool=None):
      """Create a new proxy object for svn_diff_fns_t"""
      import libsvn.core, weakref
      self.__dict__["_parent_pool"] = \
        parent_pool or libsvn.core.application_pool;
      if self.__dict__["_parent_pool"]:
        self.__dict__["_is_valid"] = weakref.ref(
          self.__dict__["_parent_pool"]._is_valid)

    def assert_valid(self):
      """Assert that this object is using valid pool memory"""
      if "_is_valid" in self.__dict__:
        assert self.__dict__["_is_valid"](), "Variable has already been deleted"

    def __getattr__(self, name):
      """Get an attribute from this object"""
      self.assert_valid()

      value = _swig_getattr(self, self.__class__, name)



      members = self.__dict__.get("_members")
      if members is not None:
        _copy_metadata_deep(value, members.get(name))
          

      _assert_valid_deep(value)

      return value

    def __setattr__(self, name, value):
      """Set an attribute on this object"""
      self.assert_valid()




      self.__dict__.setdefault("_members",{})[name] = value

      return _swig_setattr(self, self.__class__, name, value)

    def datasource_open(self, *args):
      return svn_diff_fns_invoke_datasource_open(self, *args)

    def datasource_close(self, *args):
      return svn_diff_fns_invoke_datasource_close(self, *args)

    def datasource_get_next_token(self, *args):
      return svn_diff_fns_invoke_datasource_get_next_token(self, *args)

    def token_compare(self, *args):
      return svn_diff_fns_invoke_token_compare(self, *args)

    def token_discard(self, *args):
      return svn_diff_fns_invoke_token_discard(self, *args)

    def token_discard_all(self, *args):
      return svn_diff_fns_invoke_token_discard_all(self, *args)

    def __init__(self): 
        """__init__(svn_diff_fns_t self) -> svn_diff_fns_t"""
        this = _diff.new_svn_diff_fns_t()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _diff.delete_svn_diff_fns_t
    __del__ = lambda self : None;
svn_diff_fns_t_swigregister = _diff.svn_diff_fns_t_swigregister
svn_diff_fns_t_swigregister(svn_diff_fns_t)


def svn_diff_diff_2(*args):
  """svn_diff_diff_2(void * diff_baton, svn_diff_fns2_t diff_fns, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_diff_2(*args)

def svn_diff_diff(*args):
  """svn_diff_diff(void * diff_baton, svn_diff_fns_t diff_fns, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_diff(*args)

def svn_diff_diff3_2(*args):
  """svn_diff_diff3_2(void * diff_baton, svn_diff_fns2_t diff_fns, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_diff3_2(*args)

def svn_diff_diff3(*args):
  """svn_diff_diff3(void * diff_baton, svn_diff_fns_t diff_fns, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_diff3(*args)

def svn_diff_diff4_2(*args):
  """svn_diff_diff4_2(void * diff_baton, svn_diff_fns2_t diff_fns, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_diff4_2(*args)

def svn_diff_diff4(*args):
  """svn_diff_diff4(void * diff_baton, svn_diff_fns_t diff_fns, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_diff4(*args)

def svn_diff_contains_conflicts(*args):
  """svn_diff_contains_conflicts(svn_diff_t * diff) -> svn_boolean_t"""
  return _diff.svn_diff_contains_conflicts(*args)

def svn_diff_contains_diffs(*args):
  """svn_diff_contains_diffs(svn_diff_t * diff) -> svn_boolean_t"""
  return _diff.svn_diff_contains_diffs(*args)
class svn_diff_output_fns_t:
    """Proxy of C svn_diff_output_fns_t struct"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, svn_diff_output_fns_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, svn_diff_output_fns_t, name)
    __repr__ = _swig_repr
    __swig_setmethods__["output_common"] = _diff.svn_diff_output_fns_t_output_common_set
    __swig_getmethods__["output_common"] = _diff.svn_diff_output_fns_t_output_common_get
    __swig_setmethods__["output_diff_modified"] = _diff.svn_diff_output_fns_t_output_diff_modified_set
    __swig_getmethods__["output_diff_modified"] = _diff.svn_diff_output_fns_t_output_diff_modified_get
    __swig_setmethods__["output_diff_latest"] = _diff.svn_diff_output_fns_t_output_diff_latest_set
    __swig_getmethods__["output_diff_latest"] = _diff.svn_diff_output_fns_t_output_diff_latest_get
    __swig_setmethods__["output_diff_common"] = _diff.svn_diff_output_fns_t_output_diff_common_set
    __swig_getmethods__["output_diff_common"] = _diff.svn_diff_output_fns_t_output_diff_common_get
    __swig_setmethods__["output_conflict"] = _diff.svn_diff_output_fns_t_output_conflict_set
    __swig_getmethods__["output_conflict"] = _diff.svn_diff_output_fns_t_output_conflict_get
    def set_parent_pool(self, parent_pool=None):
      """Create a new proxy object for svn_diff_output_fns_t"""
      import libsvn.core, weakref
      self.__dict__["_parent_pool"] = \
        parent_pool or libsvn.core.application_pool;
      if self.__dict__["_parent_pool"]:
        self.__dict__["_is_valid"] = weakref.ref(
          self.__dict__["_parent_pool"]._is_valid)

    def assert_valid(self):
      """Assert that this object is using valid pool memory"""
      if "_is_valid" in self.__dict__:
        assert self.__dict__["_is_valid"](), "Variable has already been deleted"

    def __getattr__(self, name):
      """Get an attribute from this object"""
      self.assert_valid()

      value = _swig_getattr(self, self.__class__, name)



      members = self.__dict__.get("_members")
      if members is not None:
        _copy_metadata_deep(value, members.get(name))
          

      _assert_valid_deep(value)

      return value

    def __setattr__(self, name, value):
      """Set an attribute on this object"""
      self.assert_valid()




      self.__dict__.setdefault("_members",{})[name] = value

      return _swig_setattr(self, self.__class__, name, value)

    def output_common(self, *args):
      return svn_diff_output_fns_invoke_output_common(self, *args)

    def output_diff_modified(self, *args):
      return svn_diff_output_fns_invoke_output_diff_modified(self, *args)

    def output_diff_latest(self, *args):
      return svn_diff_output_fns_invoke_output_diff_latest(self, *args)

    def output_diff_common(self, *args):
      return svn_diff_output_fns_invoke_output_diff_common(self, *args)

    def output_conflict(self, *args):
      return svn_diff_output_fns_invoke_output_conflict(self, *args)

    def __init__(self): 
        """__init__(svn_diff_output_fns_t self) -> svn_diff_output_fns_t"""
        this = _diff.new_svn_diff_output_fns_t()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _diff.delete_svn_diff_output_fns_t
    __del__ = lambda self : None;
svn_diff_output_fns_t_swigregister = _diff.svn_diff_output_fns_t_swigregister
svn_diff_output_fns_t_swigregister(svn_diff_output_fns_t)

svn_diff_conflict_display_modified_latest = _diff.svn_diff_conflict_display_modified_latest
svn_diff_conflict_display_resolved_modified_latest = _diff.svn_diff_conflict_display_resolved_modified_latest
svn_diff_conflict_display_modified_original_latest = _diff.svn_diff_conflict_display_modified_original_latest
svn_diff_conflict_display_modified = _diff.svn_diff_conflict_display_modified
svn_diff_conflict_display_latest = _diff.svn_diff_conflict_display_latest
svn_diff_conflict_display_only_conflicts = _diff.svn_diff_conflict_display_only_conflicts

def svn_diff_output2(*args):
  """svn_diff_output2(svn_diff_t * diff, void * output_baton, svn_diff_output_fns_t output_fns, svn_cancel_func_t cancel_func) -> svn_error_t"""
  return _diff.svn_diff_output2(*args)

def svn_diff_output(*args):
  """svn_diff_output(svn_diff_t * diff, void * output_baton, svn_diff_output_fns_t output_fns) -> svn_error_t"""
  return _diff.svn_diff_output(*args)
svn_diff_file_ignore_space_none = _diff.svn_diff_file_ignore_space_none
svn_diff_file_ignore_space_change = _diff.svn_diff_file_ignore_space_change
svn_diff_file_ignore_space_all = _diff.svn_diff_file_ignore_space_all
class svn_diff_file_options_t:
    """Proxy of C svn_diff_file_options_t struct"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, svn_diff_file_options_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, svn_diff_file_options_t, name)
    __repr__ = _swig_repr
    __swig_setmethods__["ignore_space"] = _diff.svn_diff_file_options_t_ignore_space_set
    __swig_getmethods__["ignore_space"] = _diff.svn_diff_file_options_t_ignore_space_get
    __swig_setmethods__["ignore_eol_style"] = _diff.svn_diff_file_options_t_ignore_eol_style_set
    __swig_getmethods__["ignore_eol_style"] = _diff.svn_diff_file_options_t_ignore_eol_style_get
    __swig_setmethods__["show_c_function"] = _diff.svn_diff_file_options_t_show_c_function_set
    __swig_getmethods__["show_c_function"] = _diff.svn_diff_file_options_t_show_c_function_get
    __swig_setmethods__["context_size"] = _diff.svn_diff_file_options_t_context_size_set
    __swig_getmethods__["context_size"] = _diff.svn_diff_file_options_t_context_size_get
    def set_parent_pool(self, parent_pool=None):
      """Create a new proxy object for svn_diff_file_options_t"""
      import libsvn.core, weakref
      self.__dict__["_parent_pool"] = \
        parent_pool or libsvn.core.application_pool;
      if self.__dict__["_parent_pool"]:
        self.__dict__["_is_valid"] = weakref.ref(
          self.__dict__["_parent_pool"]._is_valid)

    def assert_valid(self):
      """Assert that this object is using valid pool memory"""
      if "_is_valid" in self.__dict__:
        assert self.__dict__["_is_valid"](), "Variable has already been deleted"

    def __getattr__(self, name):
      """Get an attribute from this object"""
      self.assert_valid()

      value = _swig_getattr(self, self.__class__, name)



      members = self.__dict__.get("_members")
      if members is not None:
        _copy_metadata_deep(value, members.get(name))
          

      _assert_valid_deep(value)

      return value

    def __setattr__(self, name, value):
      """Set an attribute on this object"""
      self.assert_valid()




      self.__dict__.setdefault("_members",{})[name] = value

      return _swig_setattr(self, self.__class__, name, value)

    def __init__(self): 
        """__init__(svn_diff_file_options_t self) -> svn_diff_file_options_t"""
        this = _diff.new_svn_diff_file_options_t()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _diff.delete_svn_diff_file_options_t
    __del__ = lambda self : None;
svn_diff_file_options_t_swigregister = _diff.svn_diff_file_options_t_swigregister
svn_diff_file_options_t_swigregister(svn_diff_file_options_t)


def svn_diff_file_options_create(*args):
  """svn_diff_file_options_create(apr_pool_t pool) -> svn_diff_file_options_t"""
  return _diff.svn_diff_file_options_create(*args)

def svn_diff_file_options_parse(*args):
  """svn_diff_file_options_parse(svn_diff_file_options_t options, apr_array_header_t args, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_file_options_parse(*args)

def svn_diff_file_diff_2(*args):
  """svn_diff_file_diff_2(char const * original, char const * modified, svn_diff_file_options_t options, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_file_diff_2(*args)

def svn_diff_file_diff(*args):
  """svn_diff_file_diff(char const * original, char const * modified, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_file_diff(*args)

def svn_diff_file_diff3_2(*args):
  """
    svn_diff_file_diff3_2(char const * original, char const * modified, char const * latest, svn_diff_file_options_t options, 
        apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_file_diff3_2(*args)

def svn_diff_file_diff3(*args):
  """svn_diff_file_diff3(char const * original, char const * modified, char const * latest, apr_pool_t pool) -> svn_error_t"""
  return _diff.svn_diff_file_diff3(*args)

def svn_diff_file_diff4_2(*args):
  """
    svn_diff_file_diff4_2(char const * original, char const * modified, char const * latest, char const * ancestor, 
        svn_diff_file_options_t options, apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_file_diff4_2(*args)

def svn_diff_file_diff4(*args):
  """
    svn_diff_file_diff4(char const * original, char const * modified, char const * latest, char const * ancestor, 
        apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_file_diff4(*args)

def svn_diff_file_output_unified4(*args):
  """
    svn_diff_file_output_unified4(svn_stream_t * output_stream, svn_diff_t * diff, char const * original_path, char const * modified_path, 
        char const * original_header, char const * modified_header, 
        char const * header_encoding, char const * relative_to_dir, svn_boolean_t show_c_function, 
        int context_size, svn_cancel_func_t cancel_func, apr_pool_t scratch_pool) -> svn_error_t
    """
  return _diff.svn_diff_file_output_unified4(*args)

def svn_diff_file_output_unified3(*args):
  """
    svn_diff_file_output_unified3(svn_stream_t * output_stream, svn_diff_t * diff, char const * original_path, char const * modified_path, 
        char const * original_header, char const * modified_header, 
        char const * header_encoding, char const * relative_to_dir, svn_boolean_t show_c_function, 
        apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_file_output_unified3(*args)

def svn_diff_file_output_unified2(*args):
  """
    svn_diff_file_output_unified2(svn_stream_t * output_stream, svn_diff_t * diff, char const * original_path, char const * modified_path, 
        char const * original_header, char const * modified_header, 
        char const * header_encoding, apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_file_output_unified2(*args)

def svn_diff_file_output_unified(*args):
  """
    svn_diff_file_output_unified(svn_stream_t * output_stream, svn_diff_t * diff, char const * original_path, char const * modified_path, 
        char const * original_header, char const * modified_header, 
        apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_file_output_unified(*args)

def svn_diff_file_output_merge3(*args):
  """
    svn_diff_file_output_merge3(svn_stream_t * output_stream, svn_diff_t * diff, char const * original_path, char const * modified_path, 
        char const * latest_path, char const * conflict_original, 
        char const * conflict_modified, char const * conflict_latest, char const * conflict_separator, 
        svn_diff_conflict_display_style_t conflict_style, 
        svn_cancel_func_t cancel_func, apr_pool_t scratch_pool) -> svn_error_t
    """
  return _diff.svn_diff_file_output_merge3(*args)

def svn_diff_file_output_merge2(*args):
  """
    svn_diff_file_output_merge2(svn_stream_t * output_stream, svn_diff_t * diff, char const * original_path, char const * modified_path, 
        char const * latest_path, char const * conflict_original, 
        char const * conflict_modified, char const * conflict_latest, char const * conflict_separator, 
        svn_diff_conflict_display_style_t conflict_style, 
        apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_file_output_merge2(*args)

def svn_diff_file_output_merge(*args):
  """
    svn_diff_file_output_merge(svn_stream_t * output_stream, svn_diff_t * diff, char const * original_path, char const * modified_path, 
        char const * latest_path, char const * conflict_original, 
        char const * conflict_modified, char const * conflict_latest, char const * conflict_separator, 
        svn_boolean_t display_original_in_conflict, svn_boolean_t display_resolved_conflicts, 
        apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_file_output_merge(*args)

def svn_diff_output_binary(*args):
  """
    svn_diff_output_binary(svn_stream_t * output_stream, svn_stream_t * original, svn_stream_t * latest, svn_cancel_func_t cancel_func, 
        apr_pool_t scratch_pool) -> svn_error_t
    """
  return _diff.svn_diff_output_binary(*args)

def svn_diff_mem_string_diff(*args):
  """
    svn_diff_mem_string_diff(svn_string_t const * original, svn_string_t const * modified, svn_diff_file_options_t options, 
        apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_mem_string_diff(*args)

def svn_diff_mem_string_diff3(*args):
  """
    svn_diff_mem_string_diff3(svn_string_t const * original, svn_string_t const * modified, svn_string_t const * latest, 
        svn_diff_file_options_t options, apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_mem_string_diff3(*args)

def svn_diff_mem_string_diff4(*args):
  """
    svn_diff_mem_string_diff4(svn_string_t const * original, svn_string_t const * modified, svn_string_t const * latest, 
        svn_string_t const * ancestor, svn_diff_file_options_t options, apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_mem_string_diff4(*args)

def svn_diff_mem_string_output_unified3(*args):
  """
    svn_diff_mem_string_output_unified3(svn_stream_t * output_stream, svn_diff_t * diff, svn_boolean_t with_diff_header, 
        char const * hunk_delimiter, char const * original_header, char const * modified_header, 
        char const * header_encoding, svn_string_t const * original, 
        svn_string_t const * modified, int context_size, svn_cancel_func_t cancel_func, 
        apr_pool_t scratch_pool) -> svn_error_t
    """
  return _diff.svn_diff_mem_string_output_unified3(*args)

def svn_diff_mem_string_output_unified2(*args):
  """
    svn_diff_mem_string_output_unified2(svn_stream_t * output_stream, svn_diff_t * diff, svn_boolean_t with_diff_header, 
        char const * hunk_delimiter, char const * original_header, char const * modified_header, 
        char const * header_encoding, svn_string_t const * original, 
        svn_string_t const * modified, apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_mem_string_output_unified2(*args)

def svn_diff_mem_string_output_unified(*args):
  """
    svn_diff_mem_string_output_unified(svn_stream_t * output_stream, svn_diff_t * diff, char const * original_header, char const * modified_header, 
        char const * header_encoding, svn_string_t const * original, 
        svn_string_t const * modified, apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_mem_string_output_unified(*args)

def svn_diff_mem_string_output_merge3(*args):
  """
    svn_diff_mem_string_output_merge3(svn_stream_t * output_stream, svn_diff_t * diff, svn_string_t const * original, svn_string_t const * modified, 
        svn_string_t const * latest, char const * conflict_original, 
        char const * conflict_modified, char const * conflict_latest, 
        char const * conflict_separator, svn_diff_conflict_display_style_t style, 
        svn_cancel_func_t cancel_func, apr_pool_t scratch_pool) -> svn_error_t
    """
  return _diff.svn_diff_mem_string_output_merge3(*args)

def svn_diff_mem_string_output_merge2(*args):
  """
    svn_diff_mem_string_output_merge2(svn_stream_t * output_stream, svn_diff_t * diff, svn_string_t const * original, svn_string_t const * modified, 
        svn_string_t const * latest, char const * conflict_original, 
        char const * conflict_modified, char const * conflict_latest, 
        char const * conflict_separator, svn_diff_conflict_display_style_t style, 
        apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_mem_string_output_merge2(*args)

def svn_diff_mem_string_output_merge(*args):
  """
    svn_diff_mem_string_output_merge(svn_stream_t * output_stream, svn_diff_t * diff, svn_string_t const * original, svn_string_t const * modified, 
        svn_string_t const * latest, char const * conflict_original, 
        char const * conflict_modified, char const * conflict_latest, 
        char const * conflict_separator, svn_boolean_t display_original_in_conflict, 
        svn_boolean_t display_resolved_conflicts, apr_pool_t pool) -> svn_error_t
    """
  return _diff.svn_diff_mem_string_output_merge(*args)
svn_diff_op_unchanged = _diff.svn_diff_op_unchanged
svn_diff_op_added = _diff.svn_diff_op_added
svn_diff_op_deleted = _diff.svn_diff_op_deleted
svn_diff_op_copied = _diff.svn_diff_op_copied
svn_diff_op_moved = _diff.svn_diff_op_moved
svn_diff_op_modified = _diff.svn_diff_op_modified

def svn_diff_hunk_readline_diff_text(*args):
  """svn_diff_hunk_readline_diff_text(svn_diff_hunk_t * hunk, apr_pool_t result_pool, apr_pool_t scratch_pool) -> svn_error_t"""
  return _diff.svn_diff_hunk_readline_diff_text(*args)

def svn_diff_hunk_readline_original_text(*args):
  """svn_diff_hunk_readline_original_text(svn_diff_hunk_t * hunk, apr_pool_t result_pool, apr_pool_t scratch_pool) -> svn_error_t"""
  return _diff.svn_diff_hunk_readline_original_text(*args)

def svn_diff_hunk_readline_modified_text(*args):
  """svn_diff_hunk_readline_modified_text(svn_diff_hunk_t * hunk, apr_pool_t result_pool, apr_pool_t scratch_pool) -> svn_error_t"""
  return _diff.svn_diff_hunk_readline_modified_text(*args)

def svn_diff_hunk_reset_diff_text(*args):
  """svn_diff_hunk_reset_diff_text(svn_diff_hunk_t * hunk)"""
  return _diff.svn_diff_hunk_reset_diff_text(*args)

def svn_diff_hunk_reset_original_text(*args):
  """svn_diff_hunk_reset_original_text(svn_diff_hunk_t * hunk)"""
  return _diff.svn_diff_hunk_reset_original_text(*args)

def svn_diff_hunk_reset_modified_text(*args):
  """svn_diff_hunk_reset_modified_text(svn_diff_hunk_t * hunk)"""
  return _diff.svn_diff_hunk_reset_modified_text(*args)

def svn_diff_hunk_get_original_start(*args):
  """svn_diff_hunk_get_original_start(svn_diff_hunk_t const * hunk) -> svn_linenum_t"""
  return _diff.svn_diff_hunk_get_original_start(*args)

def svn_diff_hunk_get_original_length(*args):
  """svn_diff_hunk_get_original_length(svn_diff_hunk_t const * hunk) -> svn_linenum_t"""
  return _diff.svn_diff_hunk_get_original_length(*args)

def svn_diff_hunk_get_modified_start(*args):
  """svn_diff_hunk_get_modified_start(svn_diff_hunk_t const * hunk) -> svn_linenum_t"""
  return _diff.svn_diff_hunk_get_modified_start(*args)

def svn_diff_hunk_get_modified_length(*args):
  """svn_diff_hunk_get_modified_length(svn_diff_hunk_t const * hunk) -> svn_linenum_t"""
  return _diff.svn_diff_hunk_get_modified_length(*args)

def svn_diff_hunk_get_leading_context(*args):
  """svn_diff_hunk_get_leading_context(svn_diff_hunk_t const * hunk) -> svn_linenum_t"""
  return _diff.svn_diff_hunk_get_leading_context(*args)

def svn_diff_hunk_get_trailing_context(*args):
  """svn_diff_hunk_get_trailing_context(svn_diff_hunk_t const * hunk) -> svn_linenum_t"""
  return _diff.svn_diff_hunk_get_trailing_context(*args)
class svn_prop_patch_t:
    """Proxy of C svn_prop_patch_t struct"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, svn_prop_patch_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, svn_prop_patch_t, name)
    __repr__ = _swig_repr
    __swig_setmethods__["name"] = _diff.svn_prop_patch_t_name_set
    __swig_getmethods__["name"] = _diff.svn_prop_patch_t_name_get
    __swig_setmethods__["operation"] = _diff.svn_prop_patch_t_operation_set
    __swig_getmethods__["operation"] = _diff.svn_prop_patch_t_operation_get
    __swig_setmethods__["hunks"] = _diff.svn_prop_patch_t_hunks_set
    __swig_getmethods__["hunks"] = _diff.svn_prop_patch_t_hunks_get
    def set_parent_pool(self, parent_pool=None):
      """Create a new proxy object for svn_prop_patch_t"""
      import libsvn.core, weakref
      self.__dict__["_parent_pool"] = \
        parent_pool or libsvn.core.application_pool;
      if self.__dict__["_parent_pool"]:
        self.__dict__["_is_valid"] = weakref.ref(
          self.__dict__["_parent_pool"]._is_valid)

    def assert_valid(self):
      """Assert that this object is using valid pool memory"""
      if "_is_valid" in self.__dict__:
        assert self.__dict__["_is_valid"](), "Variable has already been deleted"

    def __getattr__(self, name):
      """Get an attribute from this object"""
      self.assert_valid()

      value = _swig_getattr(self, self.__class__, name)



      members = self.__dict__.get("_members")
      if members is not None:
        _copy_metadata_deep(value, members.get(name))
          

      _assert_valid_deep(value)

      return value

    def __setattr__(self, name, value):
      """Set an attribute on this object"""
      self.assert_valid()




      self.__dict__.setdefault("_members",{})[name] = value

      return _swig_setattr(self, self.__class__, name, value)

    def __init__(self): 
        """__init__(svn_prop_patch_t self) -> svn_prop_patch_t"""
        this = _diff.new_svn_prop_patch_t()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _diff.delete_svn_prop_patch_t
    __del__ = lambda self : None;
svn_prop_patch_t_swigregister = _diff.svn_prop_patch_t_swigregister
svn_prop_patch_t_swigregister(svn_prop_patch_t)

class svn_patch_t:
    """Proxy of C svn_patch_t struct"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, svn_patch_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, svn_patch_t, name)
    __repr__ = _swig_repr
    __swig_setmethods__["old_filename"] = _diff.svn_patch_t_old_filename_set
    __swig_getmethods__["old_filename"] = _diff.svn_patch_t_old_filename_get
    __swig_setmethods__["new_filename"] = _diff.svn_patch_t_new_filename_set
    __swig_getmethods__["new_filename"] = _diff.svn_patch_t_new_filename_get
    __swig_setmethods__["hunks"] = _diff.svn_patch_t_hunks_set
    __swig_getmethods__["hunks"] = _diff.svn_patch_t_hunks_get
    __swig_setmethods__["prop_patches"] = _diff.svn_patch_t_prop_patches_set
    __swig_getmethods__["prop_patches"] = _diff.svn_patch_t_prop_patches_get
    __swig_setmethods__["operation"] = _diff.svn_patch_t_operation_set
    __swig_getmethods__["operation"] = _diff.svn_patch_t_operation_get
    __swig_setmethods__["reverse"] = _diff.svn_patch_t_reverse_set
    __swig_getmethods__["reverse"] = _diff.svn_patch_t_reverse_get
    __swig_setmethods__["mergeinfo"] = _diff.svn_patch_t_mergeinfo_set
    __swig_getmethods__["mergeinfo"] = _diff.svn_patch_t_mergeinfo_get
    __swig_setmethods__["reverse_mergeinfo"] = _diff.svn_patch_t_reverse_mergeinfo_set
    __swig_getmethods__["reverse_mergeinfo"] = _diff.svn_patch_t_reverse_mergeinfo_get
    def set_parent_pool(self, parent_pool=None):
      """Create a new proxy object for svn_patch_t"""
      import libsvn.core, weakref
      self.__dict__["_parent_pool"] = \
        parent_pool or libsvn.core.application_pool;
      if self.__dict__["_parent_pool"]:
        self.__dict__["_is_valid"] = weakref.ref(
          self.__dict__["_parent_pool"]._is_valid)

    def assert_valid(self):
      """Assert that this object is using valid pool memory"""
      if "_is_valid" in self.__dict__:
        assert self.__dict__["_is_valid"](), "Variable has already been deleted"

    def __getattr__(self, name):
      """Get an attribute from this object"""
      self.assert_valid()

      value = _swig_getattr(self, self.__class__, name)



      members = self.__dict__.get("_members")
      if members is not None:
        _copy_metadata_deep(value, members.get(name))
          

      _assert_valid_deep(value)

      return value

    def __setattr__(self, name, value):
      """Set an attribute on this object"""
      self.assert_valid()




      self.__dict__.setdefault("_members",{})[name] = value

      return _swig_setattr(self, self.__class__, name, value)

    def __init__(self): 
        """__init__(svn_patch_t self) -> svn_patch_t"""
        this = _diff.new_svn_patch_t()
        try: self.this.append(this)
        except: self.this = this
    __swig_destroy__ = _diff.delete_svn_patch_t
    __del__ = lambda self : None;
svn_patch_t_swigregister = _diff.svn_patch_t_swigregister
svn_patch_t_swigregister(svn_patch_t)


def svn_diff_open_patch_file(*args):
  """svn_diff_open_patch_file(char const * local_abspath, apr_pool_t result_pool) -> svn_error_t"""
  return _diff.svn_diff_open_patch_file(*args)

def svn_diff_parse_next_patch(*args):
  """
    svn_diff_parse_next_patch(svn_patch_file_t * patch_file, svn_boolean_t reverse, svn_boolean_t ignore_whitespace, 
        apr_pool_t result_pool, apr_pool_t scratch_pool) -> svn_error_t
    """
  return _diff.svn_diff_parse_next_patch(*args)

def svn_diff_close_patch_file(*args):
  """svn_diff_close_patch_file(svn_patch_file_t * patch_file, apr_pool_t scratch_pool) -> svn_error_t"""
  return _diff.svn_diff_close_patch_file(*args)
class svn_diff_t:
    """Proxy of C svn_diff_t struct"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, svn_diff_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, svn_diff_t, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    def set_parent_pool(self, parent_pool=None):
      """Create a new proxy object for svn_diff_t"""
      import libsvn.core, weakref
      self.__dict__["_parent_pool"] = \
        parent_pool or libsvn.core.application_pool;
      if self.__dict__["_parent_pool"]:
        self.__dict__["_is_valid"] = weakref.ref(
          self.__dict__["_parent_pool"]._is_valid)

    def assert_valid(self):
      """Assert that this object is using valid pool memory"""
      if "_is_valid" in self.__dict__:
        assert self.__dict__["_is_valid"](), "Variable has already been deleted"

    def __getattr__(self, name):
      """Get an attribute from this object"""
      self.assert_valid()

      value = _swig_getattr(self, self.__class__, name)



      members = self.__dict__.get("_members")
      if members is not None:
        _copy_metadata_deep(value, members.get(name))
          

      _assert_valid_deep(value)

      return value

    def __setattr__(self, name, value):
      """Set an attribute on this object"""
      self.assert_valid()




      self.__dict__.setdefault("_members",{})[name] = value

      return _swig_setattr(self, self.__class__, name, value)

svn_diff_t_swigregister = _diff.svn_diff_t_swigregister
svn_diff_t_swigregister(svn_diff_t)

class svn_diff_hunk_t:
    """Proxy of C svn_diff_hunk_t struct"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, svn_diff_hunk_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, svn_diff_hunk_t, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    def set_parent_pool(self, parent_pool=None):
      """Create a new proxy object for svn_diff_hunk_t"""
      import libsvn.core, weakref
      self.__dict__["_parent_pool"] = \
        parent_pool or libsvn.core.application_pool;
      if self.__dict__["_parent_pool"]:
        self.__dict__["_is_valid"] = weakref.ref(
          self.__dict__["_parent_pool"]._is_valid)

    def assert_valid(self):
      """Assert that this object is using valid pool memory"""
      if "_is_valid" in self.__dict__:
        assert self.__dict__["_is_valid"](), "Variable has already been deleted"

    def __getattr__(self, name):
      """Get an attribute from this object"""
      self.assert_valid()

      value = _swig_getattr(self, self.__class__, name)



      members = self.__dict__.get("_members")
      if members is not None:
        _copy_metadata_deep(value, members.get(name))
          

      _assert_valid_deep(value)

      return value

    def __setattr__(self, name, value):
      """Set an attribute on this object"""
      self.assert_valid()




      self.__dict__.setdefault("_members",{})[name] = value

      return _swig_setattr(self, self.__class__, name, value)

svn_diff_hunk_t_swigregister = _diff.svn_diff_hunk_t_swigregister
svn_diff_hunk_t_swigregister(svn_diff_hunk_t)

class svn_patch_file_t:
    """Proxy of C svn_patch_file_t struct"""
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, svn_patch_file_t, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, svn_patch_file_t, name)
    def __init__(self, *args, **kwargs): raise AttributeError("No constructor defined")
    __repr__ = _swig_repr
    def set_parent_pool(self, parent_pool=None):
      """Create a new proxy object for svn_patch_file_t"""
      import libsvn.core, weakref
      self.__dict__["_parent_pool"] = \
        parent_pool or libsvn.core.application_pool;
      if self.__dict__["_parent_pool"]:
        self.__dict__["_is_valid"] = weakref.ref(
          self.__dict__["_parent_pool"]._is_valid)

    def assert_valid(self):
      """Assert that this object is using valid pool memory"""
      if "_is_valid" in self.__dict__:
        assert self.__dict__["_is_valid"](), "Variable has already been deleted"

    def __getattr__(self, name):
      """Get an attribute from this object"""
      self.assert_valid()

      value = _swig_getattr(self, self.__class__, name)



      members = self.__dict__.get("_members")
      if members is not None:
        _copy_metadata_deep(value, members.get(name))
          

      _assert_valid_deep(value)

      return value

    def __setattr__(self, name, value):
      """Set an attribute on this object"""
      self.assert_valid()




      self.__dict__.setdefault("_members",{})[name] = value

      return _swig_setattr(self, self.__class__, name, value)

svn_patch_file_t_swigregister = _diff.svn_patch_file_t_swigregister
svn_patch_file_t_swigregister(svn_patch_file_t)


def svn_diff_fns2_invoke_datasources_open(*args):
  """
    svn_diff_fns2_invoke_datasources_open(svn_diff_fns2_t _obj, void * diff_baton, apr_off_t * prefix_lines, apr_off_t * suffix_lines, 
        svn_diff_datasource_e const * datasources, apr_size_t datasources_len) -> svn_error_t
    """
  return _diff.svn_diff_fns2_invoke_datasources_open(*args)

def svn_diff_fns2_invoke_datasource_close(*args):
  """svn_diff_fns2_invoke_datasource_close(svn_diff_fns2_t _obj, void * diff_baton, svn_diff_datasource_e datasource) -> svn_error_t"""
  return _diff.svn_diff_fns2_invoke_datasource_close(*args)

def svn_diff_fns2_invoke_datasource_get_next_token(*args):
  """svn_diff_fns2_invoke_datasource_get_next_token(svn_diff_fns2_t _obj, void * diff_baton, svn_diff_datasource_e datasource) -> svn_error_t"""
  return _diff.svn_diff_fns2_invoke_datasource_get_next_token(*args)

def svn_diff_fns2_invoke_token_compare(*args):
  """svn_diff_fns2_invoke_token_compare(svn_diff_fns2_t _obj, void * diff_baton, void * ltoken, void * rtoken) -> svn_error_t"""
  return _diff.svn_diff_fns2_invoke_token_compare(*args)

def svn_diff_fns2_invoke_token_discard(*args):
  """svn_diff_fns2_invoke_token_discard(svn_diff_fns2_t _obj, void * diff_baton, void * token)"""
  return _diff.svn_diff_fns2_invoke_token_discard(*args)

def svn_diff_fns2_invoke_token_discard_all(*args):
  """svn_diff_fns2_invoke_token_discard_all(svn_diff_fns2_t _obj, void * diff_baton)"""
  return _diff.svn_diff_fns2_invoke_token_discard_all(*args)

def svn_diff_fns_invoke_datasource_open(*args):
  """svn_diff_fns_invoke_datasource_open(svn_diff_fns_t _obj, void * diff_baton, svn_diff_datasource_e datasource) -> svn_error_t"""
  return _diff.svn_diff_fns_invoke_datasource_open(*args)

def svn_diff_fns_invoke_datasource_close(*args):
  """svn_diff_fns_invoke_datasource_close(svn_diff_fns_t _obj, void * diff_baton, svn_diff_datasource_e datasource) -> svn_error_t"""
  return _diff.svn_diff_fns_invoke_datasource_close(*args)

def svn_diff_fns_invoke_datasource_get_next_token(*args):
  """svn_diff_fns_invoke_datasource_get_next_token(svn_diff_fns_t _obj, void * diff_baton, svn_diff_datasource_e datasource) -> svn_error_t"""
  return _diff.svn_diff_fns_invoke_datasource_get_next_token(*args)

def svn_diff_fns_invoke_token_compare(*args):
  """svn_diff_fns_invoke_token_compare(svn_diff_fns_t _obj, void * diff_baton, void * ltoken, void * rtoken) -> svn_error_t"""
  return _diff.svn_diff_fns_invoke_token_compare(*args)

def svn_diff_fns_invoke_token_discard(*args):
  """svn_diff_fns_invoke_token_discard(svn_diff_fns_t _obj, void * diff_baton, void * token)"""
  return _diff.svn_diff_fns_invoke_token_discard(*args)

def svn_diff_fns_invoke_token_discard_all(*args):
  """svn_diff_fns_invoke_token_discard_all(svn_diff_fns_t _obj, void * diff_baton)"""
  return _diff.svn_diff_fns_invoke_token_discard_all(*args)

def svn_diff_output_fns_invoke_output_common(*args):
  """
    svn_diff_output_fns_invoke_output_common(svn_diff_output_fns_t _obj, void * output_baton, apr_off_t original_start, apr_off_t original_length, 
        apr_off_t modified_start, apr_off_t modified_length, apr_off_t latest_start, 
        apr_off_t latest_length) -> svn_error_t
    """
  return _diff.svn_diff_output_fns_invoke_output_common(*args)

def svn_diff_output_fns_invoke_output_diff_modified(*args):
  """
    svn_diff_output_fns_invoke_output_diff_modified(svn_diff_output_fns_t _obj, void * output_baton, apr_off_t original_start, apr_off_t original_length, 
        apr_off_t modified_start, apr_off_t modified_length, apr_off_t latest_start, 
        apr_off_t latest_length) -> svn_error_t
    """
  return _diff.svn_diff_output_fns_invoke_output_diff_modified(*args)

def svn_diff_output_fns_invoke_output_diff_latest(*args):
  """
    svn_diff_output_fns_invoke_output_diff_latest(svn_diff_output_fns_t _obj, void * output_baton, apr_off_t original_start, apr_off_t original_length, 
        apr_off_t modified_start, apr_off_t modified_length, apr_off_t latest_start, 
        apr_off_t latest_length) -> svn_error_t
    """
  return _diff.svn_diff_output_fns_invoke_output_diff_latest(*args)

def svn_diff_output_fns_invoke_output_diff_common(*args):
  """
    svn_diff_output_fns_invoke_output_diff_common(svn_diff_output_fns_t _obj, void * output_baton, apr_off_t original_start, apr_off_t original_length, 
        apr_off_t modified_start, apr_off_t modified_length, apr_off_t latest_start, 
        apr_off_t latest_length) -> svn_error_t
    """
  return _diff.svn_diff_output_fns_invoke_output_diff_common(*args)

def svn_diff_output_fns_invoke_output_conflict(*args):
  """
    svn_diff_output_fns_invoke_output_conflict(svn_diff_output_fns_t _obj, void * output_baton, apr_off_t original_start, apr_off_t original_length, 
        apr_off_t modified_start, apr_off_t modified_length, apr_off_t latest_start, 
        apr_off_t latest_length, svn_diff_t * resolved_diff) -> svn_error_t
    """
  return _diff.svn_diff_output_fns_invoke_output_conflict(*args)
# This file is compatible with both classic and new-style classes.





#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.


from ctypes import *
import functions
from functions import *
from tempfile import TemporaryFile
import sys

# Constants which are defined in the Subversion headers, but aren't
# picked up by ctypesgen
TRUE = 1
FALSE = 0
SVN_DIRENT_ALL = -1
SVN_INVALID_REVNUM = -1
SVN_IGNORED_REVNUM = -1
SVN_INVALID_FILESIZE = -1
SVN_RA_SVN_UNSPECIFIED_NUMBER = -1

# Convert the standard files
stdout = PyFile_AsFile(sys.stdout)
stderr = PyFile_AsFile(sys.stderr)
stdin = PyFile_AsFile(sys.stdin)

def svn_pool_create(pool):
    return svn_pool_create_ex(pool, NULL)

def svn_pool_destroy(pool):
    return apr_pool_destroy(pool)

def svn_pool_clear(pool):
    return apr_pool_clear(pool)

def _mark_weakpool_invalid(weakpool):
  if weakpool and weakpool() and hasattr(weakpool(), "_is_valid"):
    del weakpool()._is_valid

class Pool(object):
  def __init__(self, parent_pool=None):
    """Create a new memory pool"""
    self._parent_pool = parent_pool
    self._as_parameter_ = svn_pool_create(self._parent_pool)
    self._mark_valid()

    # Protect important functions from GC
    self._svn_pool_destroy = apr_pool_destroy

  def valid(self):
    """Check whether this memory pool and its parents
    are still valid"""
    return hasattr(self,"_is_valid")

  def assert_valid(self):
    """Assert that this memory_pool is still valid."""
    assert self.valid(), "This pool has already been destroyed"

  def clear(self):
    """Clear embedded memory pool. Invalidate all subpools."""
    pool = self._parent_pool
    svn_pool_clear(self)
    self._mark_valid()

  def destroy(self):
    """Destroy embedded memory pool. If you do not destroy
    the memory pool manually, Python will destroy it
    automatically."""

    self.assert_valid()

    # Destroy pool
    self._svn_pool_destroy(self)

    # Mark self as invalid
    if hasattr(self, "_parent_pool"):
      del self._parent_pool
    if hasattr(self, "_is_valid"):
      del self._is_valid

  def __del__(self):
    """Automatically destroy memory pools, if necessary"""
    if self.valid():
      self.destroy()

  def _mark_valid(self):
    """Mark pool as valid"""

    self._weakparent = None

    if self._parent_pool:
      import weakref

      # Make sure that the parent object is valid
      self._parent_pool.assert_valid()

      # Refer to self using a weakreference so that we don't
      # create a reference cycle
      weakself = weakref.ref(self)

      # Set up callbacks to mark pool as invalid when parents
      # are destroyed
      weakparent = weakref.ref(self._parent_pool._is_valid,
        lambda x: _mark_weakpool_invalid(weakself))

    # Mark pool as valid
    self._is_valid = lambda: 1


# Special case for Subversion 1.5 and earlier: Make sure that platform-
# specific functions are available, even if the headers were generated on a
# different platform.

for f in ('svn_auth_get_windows_simple_provider',
          'svn_auth_get_windows_ssl_server_trust_provider',
          'svn_auth_get_keychain_simple_provider'):

    if hasattr(functions._libs['svn_subr-1'], f) and f not in locals():
        f = getattr(functions._libs['svn_subr-1'], f)
        f.restype = None
        f.argtypes = [POINTER(POINTER(svn_auth_provider_object_t)), POINTER(apr_pool_t)]




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# which-error.py: Print semantic Subversion error code names mapped from
#                 their numeric error code values
#
# ====================================================================
#    Licensed to the Apache Software Foundation (ASF) under one
#    or more contributor license agreements.  See the NOTICE file
#    distributed with this work for additional information
#    regarding copyright ownership.  The ASF licenses this file
#    to you under the Apache License, Version 2.0 (the
#    "License"); you may not use this file except in compliance
#    with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing,
#    software distributed under the License is distributed on an
#    "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
#    KIND, either express or implied.  See the License for the
#    specific language governing permissions and limitations
#    under the License.
# ====================================================================
#
# $HeadURL: http://svn.apache.org/repos/asf/subversion/branches/1.9.x/tools/dev/which-error.py $
# $LastChangedDate: 2013-12-05 00:42:34 +0000 (Thu, 05 Dec 2013) $
# $LastChangedBy: breser $
# $LastChangedRevision: 1547977 $
#

import errno
import sys
import os.path
import re

try:
  from svn import core
except ImportError, e:
  sys.stderr.write("ERROR: Unable to import Subversion's Python bindings: '%s'\n" \
                   "Hint: Set your PYTHONPATH environment variable, or adjust your " \
                   "PYTHONSTARTUP\nfile to point to your Subversion install " \
                   "location's svn-python directory.\n" % e)
  sys.stderr.flush()
  sys.exit(1)


def usage_and_exit():
  progname = os.path.basename(sys.argv[0])
  sys.stderr.write("""Usage: 1. %s ERRNUM [...]
       2. %s parse
       3. %s list

Print numeric and semantic error code information for Subversion error
codes.  This can be done in variety of ways:

   1. For each ERRNUM, list the error code information.

   2. Parse standard input as if it was error stream from a debug-mode
      Subversion command-line client, echoing that input to stdout,
      followed by the error code information for codes found in use in
      that error stream.

   3. Simply list the error code information for all known such
      mappings.

""" % (progname, progname, progname))
  sys.exit(1)

def get_errors():
  errs = {}
  ## errno values.
  errs.update(errno.errorcode)
  ## APR-defined errors, from apr_errno.h.
  dirname = os.path.dirname(os.path.realpath(__file__))
  for line in open(os.path.join(dirname, 'aprerr.txt')):
    # aprerr.txt parsing duplicated in gen_base.py:write_errno_table()
    if line.startswith('#'):
       continue
    key, _, val = line.split()
    errs[int(val)] = key
  ## Subversion errors, from svn_error_codes.h.
  for key in vars(core):
    if key.find('SVN_ERR_') == 0:
      try:
        val = int(vars(core)[key])
        errs[val] = key
      except:
        pass
  return errs

def print_error(code):
  try:
    print('%08d  %s' % (code, __svn_error_codes[code]))
  except KeyError:
    if code == -41:
      print("Sit by a lake.")
    else:
      print('%08d  *** UNKNOWN ERROR CODE ***' % (code))

if __name__ == "__main__":
  global __svn_error_codes
  __svn_error_codes = get_errors()
  codes = []
  if len(sys.argv) < 2:
    usage_and_exit()

  # Get a list of known codes
  if sys.argv[1] == 'list':
    if len(sys.argv) > 2:
      usage_and_exit()
    codes = sorted(__svn_error_codes.keys())

  # Get a list of code by parsing stdin for apr_err=CODE instances
  elif sys.argv[1] == 'parse':
    if len(sys.argv) > 2:
      usage_and_exit()
    while True:
      line = sys.stdin.readline()
      if not line:
        break
      sys.stdout.write(line)
      match = re.match(r'^.*apr_err=([0-9]+)[^0-9].*$', line)
      if match:
        codes.append(int(match.group(1)))

  # Get the list of requested codes
  else:
    for code in sys.argv[1:]:
      try:
        code = code.lstrip('EW')
        codes.append(int(code))
      except ValueError:
        usage_and_exit()

  # Print the harvest codes
  for code in codes:
    print_error(code)


